import React, { useEffect, useState, useMemo } from "react";
import { Typography, Card, CardContent, Stack } from "@mui/material";
import {
  KeyValueTable,
  SimpleAccordion,
  AlertBannerList,
  ExternalLink,
  InternalLink,
  formatDateAndTime,
  EmptyValue,
  Duration
} from "@ess-ics/ce-ui-common";
import { JobBadge } from "./JobBadge";
import { DeploymentJobOutput } from "../deployments/DeploymentJobOutput";
import GitRefLink from "../IOC/GitRefLink";
import AccessControl from "../auth/AccessControl";
import { AWXJobDetails } from "../../api/DataTypes";
import { JobStatusStepper } from "./JobStatus";

function createAlert(operation, job) {
  const jobDetails = new AWXJobDetails(operation.type, job);
  const message = jobDetails.message();
  const severity = jobDetails.severity();
  return {
    type: severity,
    message: message
  };
}

export function JobDetails({ operation, job }) {
  let jobAlert = useMemo(() => operation.alerts ?? [], [operation]);

  const finishedJob = useMemo(
    () => operation && new AWXJobDetails(operation.type, job).isFinished(),
    [operation, job]
  );

  const [alert, setAlert] = useState(createAlert(operation, job));

  useEffect(() => {
    setAlert(createAlert(operation, job, finishedJob));
  }, [finishedJob, operation, job]);

  function calculateHostText() {
    // host is resolvable => show link for users
    if (operation.host.hostId && operation.host.externalIdValid) {
      return (
        <InternalLink
          to={`/hosts/${operation.host.hostId}`}
          label={`Host Details, ${operation.host.fqdn}`}
        >
          {operation.host.fqdn}
        </InternalLink>
      );
    }

    // gray out hostname if it is from cache
    return <Typography color="gray">{operation.host.fqdn}</Typography>;
  }

  const deploymentDetails = {
    Revision: (
      <GitRefLink
        url={operation.gitProjectUrl}
        displayReference={operation.gitShortReference}
        revision={operation.gitShortReference}
      />
    ),
    host: calculateHostText(),
    user: (
      <InternalLink
        to={`/user/${operation.createdBy}`}
        label={`User profile, ${operation.createdBy}`}
      >
        {operation.createdBy}
      </InternalLink>
    ),
    "AWX job link": (
      <ExternalLink
        href={operation.awxJobUrl}
        label="AWX job"
      >
        {operation.awxJobUrl}
      </ExternalLink>
    ),
    "created time": operation?.createdAt ? (
      formatDateAndTime(operation.createdAt)
    ) : (
      <EmptyValue />
    ),
    "AWX job start time": operation?.startTime ? (
      formatDateAndTime(operation.startTime)
    ) : (
      <EmptyValue />
    ),
    duration: operation?.finishedAt ? (
      <Duration
        createOrStartDate={operation?.startTime ?? operation?.createdAt}
        finishedAt={operation?.finishedAt}
        textOnly
      />
    ) : (
      <EmptyValue />
    )
  };

  const finishedJobAlerts = finishedJob ? (
    <AlertBannerList alerts={[alert].concat(jobAlert)} />
  ) : null;

  const unFinishedJobsWithAlerts =
    !finishedJob && alert ? <AlertBannerList alerts={[alert]} /> : null;

  return (
    <Stack spacing={2}>
      <Stack>
        {finishedJobAlerts}
        {unFinishedJobsWithAlerts}
      </Stack>
      <JobBadge operation={operation} />
      <KeyValueTable
        obj={deploymentDetails}
        variant="overline"
        sx={{ border: 0 }}
        valueOptions={{ headerName: "" }}
      />
      {job ? (
        <Card>
          <CardContent>
            <JobStatusStepper {...{ job, operation }} />
          </CardContent>
        </Card>
      ) : null}
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        {job ? (
          <SimpleAccordion summary="Ansible Job Output">
            <DeploymentJobOutput deploymentJob={job} />
          </SimpleAccordion>
        ) : (
          <></>
        )}
      </AccessControl>
    </Stack>
  );
}
