import React, { useEffect, useCallback, useContext, useMemo } from "react";
import { Box, IconButton, Typography, Stack } from "@mui/material";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import { HostBadge } from "../../../components/host/HostBadge";
import {
  KeyValueTable,
  SimpleAccordion,
  GlobalAppBarContext,
  useAPIMethod,
  AlertBannerList,
  usePagination,
  ExternalLink
} from "@ess-ics/ce-ui-common";
import { LokiPanel } from "../../../components/common/Loki/LokiPanel";
import { useNavigate } from "react-router-dom";
import {
  applicationTitle,
  initRequestParams
} from "../../../components/common/Helper";
import AccessControl from "../../../components/auth/AccessControl";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../../components/common/URLState/URLState";
import { apiContext } from "../../../api/DeployApi";
import IOCTable from "../../../components/IOC/IOCTable";
import { HostDetailsTable } from "./HostDetailsTable";

export function HostDetailsView({ hostId, host }) {
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => {
    if (host && host.netBoxHost) {
      setTitle(applicationTitle("Host Details: " + host.netBoxHost.name));
    }
  }, [host, setTitle]);

  const client = useContext(apiContext);

  const {
    value: iocs,
    wrapper: getIocs,
    loading,
    dataReady,
    abort: abortGetIocs
  } = useAPIMethod({
    fcn: client.apis.Hosts.findAssociatedIocsByHostId,
    call: false
  });

  const [urlState, setUrlState] = useUrlState(
    {
      iocs_rows: "20",
      iocs_page: "0",
      iocs_open: "true",
      syslog_open: "true",
      details_open: "false"
    },
    { navigateMode: "replace" }
  );

  const navigate = useNavigate();

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.iocs_rows),
      page: deserialize(urlState.iocs_page)
    };
  }, [urlState]);

  const setUrlPagination = useCallback(
    ({ rows, page }) => {
      setUrlState({
        iocs_rows: serialize(rows),
        iocs_page: serialize(page)
      });
    },
    [setUrlState]
  );

  const rowsPerPage = [20, 50];

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: urlPagination.iocs_rows ?? rowsPerPage[0],
    initPage: urlPagination.iocs_page ?? 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: iocs?.totalCount ?? 0 });
  }, [setPagination, iocs?.totalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
    abortGetIocs();
  };

  useEffect(() => {
    let requestParams = initRequestParams(urlPagination, null);

    requestParams.host_id = hostId;

    getIocs(requestParams);

    return () => {
      abortGetIocs();
    };
  }, [getIocs, urlPagination, hostId, abortGetIocs]);

  const handleClick = (event) => {
    navigate(-1);
  };

  return (
    <Stack gap={2}>
      <Box>
        <IconButton
          color="inherit"
          onClick={handleClick}
          size="large"
        >
          <ArrowBackIcon />
        </IconButton>
      </Box>
      {host ? (
        <>
          <AlertBannerList alerts={host?.alerts ?? []} />
          <HostBadge host={host} />
        </>
      ) : null}
      <Stack gap={2}>
        <Typography variant="h3">IOCs</Typography>
        <IOCTable
          iocs={iocs?.deployedIocs}
          loading={loading || !dataReady}
          rowType="host"
          pagination={pagination}
          onPage={onPage}
        />
      </Stack>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        {host ? (
          <SimpleAccordion
            summary="Host details"
            expanded={deserialize(urlState.details_open)}
            onChange={(event, expanded) =>
              setUrlState({ details_open: serialize(expanded) })
            }
            sx={{ marginTop: "0 !important" }}
          >
            <HostDetailsTable host={host} />
          </SimpleAccordion>
        ) : null}
      </AccessControl>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <Stack gap={2}>
          <Typography variant="h3">Host log stream</Typography>
          {host ? (
            <LokiPanel
              host={host}
              isSyslog
              isDeployed
            />
          ) : null}
        </Stack>
      </AccessControl>
      <KeyValueTable
        obj={{
          "Host Configuration": (
            <ExternalLink
              href={
                host?.netBoxHost.vm
                  ? `${window.NETBOX_ADDRESS}/virtualization/virtual-machines/${host?.netBoxHost.id}`
                  : `${window.NETBOX_ADDRESS}/dcim/devices/${host?.netBoxHost.id}`
              }
              aria-label="Host Configuration"
            >
              {" "}
              {host?.netBoxHost.vm
                ? `${window.NETBOX_ADDRESS}/virtualization/virtual-machines/${host?.netBoxHost.id}`
                : `${window.NETBOX_ADDRESS}/dcim/devices/${host?.netBoxHost.id}`}
            </ExternalLink>
          ),
          "Host Metrics": (
            <ExternalLink
              href={`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.netBoxHost.fqdn}`}
              aria-label="Host Metrics"
            >
              {`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.netBoxHost.fqdn}`}
            </ExternalLink>
          )
        }}
        variant="overline"
      />
    </Stack>
  );
}
